@@ -8,12 +8,10 @@ from mindsdb_native.libs.data_types.mindsdb_logger import log
 class ClickhouseDS(SQLDataSource):
     def __init__(self, query, host='localhost', user='default', password=None,
                  port=8123, protocol='http'):
-    
+
         if ' format ' in query.lower():
             raise Exception('Please refrain from adding a "FORMAT" statement to the query')
 
-        query = '{} FORMAT JSON'.format(query.rstrip(" ;\n"))
-
         super().__init__(query)
 
         self.host = host
@@ -26,6 +24,7 @@ class ClickhouseDS(SQLDataSource):
             raise ValueError('Unexpected protocol {}'.fomat(protocol))
 
     def query(self, q):
+        q = '{} FORMAT JSON'.format(q.rstrip(" ;\n"))
         params = {'user': self.user}
         if self.password is not None:
             params['password'] = self.password
@@ -46,4 +45,4 @@ class ClickhouseDS(SQLDataSource):
         return df, self._make_colmap(df)
 
     def name(self):
-        return '{}'.format(self.__class__.__name__)
+        return 'Clickhouse - {}'.format(self._query)
