@@ -38,7 +38,4 @@ class PostgresDS(SQLDataSource):
         return df, self._make_colmap(df)
 
     def name(self):
-        return '{}: {}'.format(
-            self.__class__.__name__,
-            self.database
-        )
+        return 'Postgres - {}'.format(self._query)
