@@ -160,10 +160,20 @@ class ModelAnalyzer(BaseModule):
             )
 
         for col in output_columns:
-            acc_stats = AccStats(col_stats=self.transaction.lmd['stats_v2'][col], col_name=col, input_columns=input_columns)
+            acc_stats = AccStats(
+                col_stats=self.transaction.lmd['stats_v2'][col],
+                col_name=col,
+                input_columns=input_columns
+            )
+
             predictions_arr = [normal_predictions_test] + [x for x in empty_input_predictions_test.values()]
 
-            acc_stats.fit(test_df, predictions_arr, [[ignored_column] for ignored_column in empty_input_predictions_test])
+            acc_stats.fit(
+                test_df,
+                predictions_arr,
+                [[ignored_column] for ignored_column in empty_input_predictions_test]
+            )
+
             overall_accuracy, accuracy_histogram, cm, accuracy_samples = acc_stats.get_accuracy_stats()
             overall_accuracy_arr.append(overall_accuracy)
 
@@ -172,7 +182,9 @@ class ModelAnalyzer(BaseModule):
             self.transaction.lmd['accuracy_samples'][col] = accuracy_samples
             self.transaction.hmd['acc_stats'][col] = pickle_obj(acc_stats)
 
-        self.transaction.lmd['validation_set_accuracy'] = sum(overall_accuracy_arr) / len(overall_accuracy_arr)
+        self.transaction.lmd['validation_set_accuracy'] = normal_accuracy
+        if self.transaction.lmd['stats_v2'][col]['typing']['data_type'] == DATA_TYPES.NUMERIC:
+            self.transaction.lmd['validation_set_accuracy_r2'] = normal_accuracy
 
         # conformal prediction confidence estimation
         self.transaction.lmd['stats_v2']['train_std_dev'] = {}
@@ -246,7 +258,13 @@ class ModelAnalyzer(BaseModule):
                 if not is_classification:
                     self.transaction.lmd['stats_v2']['train_std_dev'][target] = self.transaction.input_data.train_df[target].std()
 
-                X = clean_df(X, self.transaction.lmd['stats_v2'], output_columns, fit_params['columns_to_ignore'])
+                X = clean_df(
+                    X,
+                    self.transaction.lmd['stats_v2'],
+                    output_columns,
+                    fit_params['columns_to_ignore']
+                )
+
                 self.transaction.hmd['icp'][target].index = X.columns
                 self.transaction.hmd['icp'][target].fit(X.values, y.values)
                 self.transaction.hmd['icp']['active'] = True
@@ -263,7 +281,13 @@ class ModelAnalyzer(BaseModule):
                         y = np.array([cats.index(i) for i in y])
                     y = y.astype(int)
 
-                X = clean_df(X, self.transaction.lmd['stats_v2'], output_columns, fit_params['columns_to_ignore'])
+                X = clean_df(
+                    X,
+                    self.transaction.lmd['stats_v2'],
+                    output_columns,
+                    fit_params['columns_to_ignore']
+                )
+
                 self.transaction.hmd['icp'][target].calibrate(X.values, y)
 
 
